@@ -83,7 +83,7 @@ struct msg
     WS_PROXY_MESSAGE_CALLBACK_CONTEXT   ctx_send;
     WS_PROXY_MESSAGE_CALLBACK_CONTEXT   ctx_receive;
     ULONG                               prop_count;
-    struct prop                         prop[sizeof(msg_props)/sizeof(msg_props[0])];
+    struct prop                         prop[ARRAY_SIZE( msg_props )];
 };
 
 #define MSG_MAGIC (('M' << 24) | ('E' << 16) | ('S' << 8) | 'S')
@@ -91,7 +91,7 @@ struct msg
 
 static struct msg *alloc_msg(void)
 {
-    static const ULONG count = sizeof(msg_props)/sizeof(msg_props[0]);
+    static const ULONG count = ARRAY_SIZE( msg_props );
     struct msg *ret;
     ULONG size = sizeof(*ret) + prop_size( msg_props, count );
 
@@ -106,7 +106,9 @@ static struct msg *alloc_msg(void)
     ret->header_size = HEADER_ARRAY_SIZE;
 
     InitializeCriticalSection( &ret->cs );
+#ifndef __MINGW32__
     ret->cs.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": msg.cs");
+#endif
 
     prop_init( msg_props, count, ret->prop, &ret[1] );
     ret->prop_count  = count;
@@ -162,7 +164,9 @@ static void free_msg( struct msg *msg )
     WsFreeHeap( msg->heap );
     heap_free( msg->header );
 
+#ifndef __MINGW32__
     msg->cs.DebugInfo->Spare[0] = 0;
+#endif
     DeleteCriticalSection( &msg->cs );
     heap_free( msg );
 }
@@ -290,6 +294,7 @@ void WINAPI WsFreeMessage( WS_MESSAGE *handle )
 HRESULT WINAPI WsResetMessage( WS_MESSAGE *handle, WS_ERROR *error )
 {
     struct msg *msg = (struct msg *)handle;
+    HRESULT hr = S_OK;
 
     TRACE( "%p %p\n", handle, error );
     if (error) FIXME( "ignoring error parameter\n" );
@@ -307,7 +312,8 @@ HRESULT WINAPI WsResetMessage( WS_MESSAGE *handle, WS_ERROR *error )
     reset_msg( msg );
 
     LeaveCriticalSection( &msg->cs );
-    return S_OK;
+    TRACE( "returning %08x\n", hr );
+    return hr;
 }
 
 /**************************************************************************
@@ -379,6 +385,7 @@ HRESULT WINAPI WsGetMessageProperty( WS_MESSAGE *handle, WS_MESSAGE_PROPERTY_ID
     }
 
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -419,6 +426,7 @@ HRESULT WINAPI WsSetMessageProperty( WS_MESSAGE *handle, WS_MESSAGE_PROPERTY_ID
     }
 
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -448,25 +456,23 @@ HRESULT WINAPI WsAddressMessage( WS_MESSAGE *handle, const WS_ENDPOINT_ADDRESS *
         return E_INVALIDARG;
     }
 
-    if (msg->state < WS_MESSAGE_STATE_INITIALIZED || msg->is_addressed)
+    if (msg->state < WS_MESSAGE_STATE_INITIALIZED || msg->is_addressed) hr = WS_E_INVALID_OPERATION;
+    else
     {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
-    }
-
-    if (addr && addr->url.length)
-    {
-        if (!(msg->addr.chars = heap_alloc( addr->url.length * sizeof(WCHAR) ))) hr = E_OUTOFMEMORY;
-        else
+        if (addr && addr->url.length)
         {
-            memcpy( msg->addr.chars, addr->url.chars, addr->url.length * sizeof(WCHAR) );
-            msg->addr.length = addr->url.length;
+            if (!(msg->addr.chars = heap_alloc( addr->url.length * sizeof(WCHAR) ))) hr = E_OUTOFMEMORY;
+            else
+            {
+                memcpy( msg->addr.chars, addr->url.chars, addr->url.length * sizeof(WCHAR) );
+                msg->addr.length = addr->url.length;
+            }
         }
     }
-
     if (hr == S_OK) msg->is_addressed = TRUE;
 
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -739,20 +745,16 @@ HRESULT WINAPI WsWriteEnvelopeStart( WS_MESSAGE *handle, WS_XML_WRITER *writer,
         return E_INVALIDARG;
     }
 
-    if (msg->state != WS_MESSAGE_STATE_INITIALIZED)
+    if (msg->state != WS_MESSAGE_STATE_INITIALIZED) hr = WS_E_INVALID_OPERATION;
+    else if ((hr = write_envelope( msg )) == S_OK &&
+             (hr = write_envelope_start( msg, writer )) == S_OK)
     {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
+        msg->writer_body = writer;
+        msg->state       = WS_MESSAGE_STATE_WRITING;
     }
 
-    if ((hr = write_envelope( msg )) != S_OK) goto done;
-    if ((hr = write_envelope_start( msg, writer )) != S_OK) goto done;
-
-    msg->writer_body = writer;
-    msg->state       = WS_MESSAGE_STATE_WRITING;
-
-done:
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -777,15 +779,12 @@ HRESULT WINAPI WsWriteEnvelopeEnd( WS_MESSAGE *handle, WS_ERROR *error )
         return E_INVALIDARG;
     }
 
-    if (msg->state != WS_MESSAGE_STATE_WRITING)
-    {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
-    }
-
-    if ((hr = write_envelope_end( msg->writer_body )) == S_OK) msg->state = WS_MESSAGE_STATE_DONE;
+    if (msg->state != WS_MESSAGE_STATE_WRITING) hr = WS_E_INVALID_OPERATION;
+    else if ((hr = write_envelope_end( msg->writer_body )) == S_OK)
+        msg->state = WS_MESSAGE_STATE_DONE;
 
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -813,8 +812,8 @@ HRESULT WINAPI WsWriteBody( WS_MESSAGE *handle, const WS_ELEMENT_DESCRIPTION *de
 
     if (msg->state != WS_MESSAGE_STATE_WRITING)
     {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
+        hr = WS_E_INVALID_OPERATION;
+        goto done;
     }
 
     if (desc->elementLocalName &&
@@ -828,6 +827,7 @@ HRESULT WINAPI WsWriteBody( WS_MESSAGE *handle, const WS_ELEMENT_DESCRIPTION *de
 
 done:
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -913,20 +913,16 @@ HRESULT WINAPI WsReadEnvelopeStart( WS_MESSAGE *handle, WS_XML_READER *reader, W
         return E_INVALIDARG;
     }
 
-    if (msg->state != WS_MESSAGE_STATE_EMPTY)
-    {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
-    }
-
-    if ((hr = read_envelope_start( msg, reader )) == S_OK &&
-        (hr = create_header_buffer( reader, msg->heap, &msg->buf )) == S_OK)
+    if (msg->state != WS_MESSAGE_STATE_EMPTY) hr = WS_E_INVALID_OPERATION;
+    else if ((hr = read_envelope_start( msg, reader )) == S_OK &&
+             (hr = create_header_buffer( reader, msg->heap, &msg->buf )) == S_OK)
     {
         msg->reader_body = reader;
         msg->state       = WS_MESSAGE_STATE_READING;
     }
 
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -958,15 +954,12 @@ HRESULT WINAPI WsReadEnvelopeEnd( WS_MESSAGE *handle, WS_ERROR *error )
         return E_INVALIDARG;
     }
 
-    if (msg->state != WS_MESSAGE_STATE_READING)
-    {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
-    }
-
-    if ((hr = read_envelope_end( msg->reader_body )) == S_OK) msg->state = WS_MESSAGE_STATE_DONE;
+    if (msg->state != WS_MESSAGE_STATE_READING) hr = WS_E_INVALID_OPERATION;
+    else if ((hr = read_envelope_end( msg->reader_body )) == S_OK)
+        msg->state = WS_MESSAGE_STATE_DONE;
 
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -992,15 +985,11 @@ HRESULT WINAPI WsReadBody( WS_MESSAGE *handle, const WS_ELEMENT_DESCRIPTION *des
         return E_INVALIDARG;
     }
 
-    if (msg->state != WS_MESSAGE_STATE_READING)
-    {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
-    }
-
-    hr = WsReadElement( msg->reader_body, desc, option, heap, value, size, NULL );
+    if (msg->state != WS_MESSAGE_STATE_READING) hr = WS_E_INVALID_OPERATION;
+    else hr = WsReadElement( msg->reader_body, desc, option, heap, value, size, NULL );
 
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -1031,19 +1020,15 @@ HRESULT WINAPI WsInitializeMessage( WS_MESSAGE *handle, WS_MESSAGE_INITIALIZATIO
         return E_INVALIDARG;
     }
 
-    if (msg->state >= WS_MESSAGE_STATE_INITIALIZED)
-    {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
-    }
-
-    if ((hr = write_envelope( msg )) == S_OK)
+    if (msg->state >= WS_MESSAGE_STATE_INITIALIZED) hr = WS_E_INVALID_OPERATION;
+    else if ((hr = write_envelope( msg )) == S_OK)
     {
         msg->init  = init;
         msg->state = WS_MESSAGE_STATE_INITIALIZED;
     }
 
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -1160,8 +1145,8 @@ HRESULT WINAPI WsSetHeader( WS_MESSAGE *handle, WS_HEADER_TYPE type, WS_TYPE val
 
     if (msg->state < WS_MESSAGE_STATE_INITIALIZED)
     {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
+        hr = WS_E_INVALID_OPERATION;
+        goto done;
     }
 
     for (i = 0; i < msg->header_count; i++)
@@ -1190,6 +1175,7 @@ HRESULT WINAPI WsSetHeader( WS_MESSAGE *handle, WS_HEADER_TYPE type, WS_TYPE val
 
 done:
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -1252,15 +1238,11 @@ HRESULT WINAPI WsGetHeader( WS_MESSAGE *handle, WS_HEADER_TYPE type, WS_TYPE val
         return E_INVALIDARG;
     }
 
-    if (msg->state < WS_MESSAGE_STATE_INITIALIZED)
-    {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
-    }
-
-    hr = get_standard_header( msg, type, value_type, option, heap, value, size );
+    if (msg->state < WS_MESSAGE_STATE_INITIALIZED) hr = WS_E_INVALID_OPERATION;
+    else hr = get_standard_header( msg, type, value_type, option, heap, value, size );
 
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -1294,31 +1276,24 @@ HRESULT WINAPI WsRemoveHeader( WS_MESSAGE *handle, WS_HEADER_TYPE type, WS_ERROR
         return E_INVALIDARG;
     }
 
-    if (msg->state < WS_MESSAGE_STATE_INITIALIZED)
+    if (msg->state < WS_MESSAGE_STATE_INITIALIZED) hr = WS_E_INVALID_OPERATION;
+    else if (type < WS_ACTION_HEADER || type > WS_FAULT_TO_HEADER) hr = E_INVALIDARG;
+    else
     {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
-    }
-
-    if (type < WS_ACTION_HEADER || type > WS_FAULT_TO_HEADER)
-    {
-        LeaveCriticalSection( &msg->cs );
-        return E_INVALIDARG;
-    }
-
-    for (i = 0; i < msg->header_count; i++)
-    {
-        if (msg->header[i]->type == type)
+        for (i = 0; i < msg->header_count; i++)
         {
-            remove_header( msg, i );
-            removed = TRUE;
-            break;
+            if (msg->header[i]->type == type)
+            {
+                remove_header( msg, i );
+                removed = TRUE;
+                break;
+            }
         }
+        if (removed) hr = write_envelope( msg );
     }
 
-    if (removed) hr = write_envelope( msg );
-
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -1436,8 +1411,8 @@ HRESULT WINAPI WsAddMappedHeader( WS_MESSAGE *handle, const WS_XML_STRING *name,
 
     if (msg->state < WS_MESSAGE_STATE_INITIALIZED)
     {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
+        hr = WS_E_INVALID_OPERATION;
+        goto done;
     }
 
     for (i = 0; i < msg->header_count; i++)
@@ -1465,6 +1440,7 @@ HRESULT WINAPI WsAddMappedHeader( WS_MESSAGE *handle, const WS_XML_STRING *name,
 
 done:
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -1474,6 +1450,7 @@ done:
 HRESULT WINAPI WsRemoveMappedHeader( WS_MESSAGE *handle, const WS_XML_STRING *name, WS_ERROR *error )
 {
     struct msg *msg = (struct msg *)handle;
+    HRESULT hr = S_OK;
     ULONG i;
 
     TRACE( "%p %s %p\n", handle, debugstr_xmlstr(name), error );
@@ -1489,24 +1466,23 @@ HRESULT WINAPI WsRemoveMappedHeader( WS_MESSAGE *handle, const WS_XML_STRING *na
         return E_INVALIDARG;
     }
 
-    if (msg->state < WS_MESSAGE_STATE_INITIALIZED)
+    if (msg->state < WS_MESSAGE_STATE_INITIALIZED) hr = WS_E_INVALID_OPERATION;
+    else
     {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
-    }
-
-    for (i = 0; i < msg->header_count; i++)
-    {
-        if (msg->header[i]->type || !msg->header[i]->mapped) continue;
-        if (WsXmlStringEquals( name, &msg->header[i]->name, NULL ) == S_OK)
+        for (i = 0; i < msg->header_count; i++)
         {
-            remove_header( msg, i );
-            break;
+            if (msg->header[i]->type || !msg->header[i]->mapped) continue;
+            if (WsXmlStringEquals( name, &msg->header[i]->name, NULL ) == S_OK)
+            {
+                remove_header( msg, i );
+                break;
+            }
         }
     }
 
     LeaveCriticalSection( &msg->cs );
-    return S_OK;
+    TRACE( "returning %08x\n", hr );
+    return hr;
 }
 
 static HRESULT write_custom_header( WS_XML_WRITER *writer, const WS_XML_STRING *name, const WS_XML_STRING *ns,
@@ -1568,8 +1544,8 @@ HRESULT WINAPI WsAddCustomHeader( WS_MESSAGE *handle, const WS_ELEMENT_DESCRIPTI
 
     if (msg->state < WS_MESSAGE_STATE_INITIALIZED)
     {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
+        hr = WS_E_INVALID_OPERATION;
+        goto done;
     }
 
     if ((hr = grow_header_array( msg, msg->header_count + 1 )) != S_OK) goto done;
@@ -1580,6 +1556,7 @@ HRESULT WINAPI WsAddCustomHeader( WS_MESSAGE *handle, const WS_ELEMENT_DESCRIPTI
 
 done:
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -1632,15 +1609,11 @@ HRESULT WINAPI WsGetCustomHeader( WS_MESSAGE *handle, const WS_ELEMENT_DESCRIPTI
         return E_INVALIDARG;
     }
 
-    if (msg->state < WS_MESSAGE_STATE_INITIALIZED)
-    {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
-    }
-
-    hr = get_custom_header( msg, desc, option, heap, value, size );
+    if (msg->state < WS_MESSAGE_STATE_INITIALIZED) hr = WS_E_INVALID_OPERATION;
+    else hr = get_custom_header( msg, desc, option, heap, value, size );
 
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -1668,27 +1641,25 @@ HRESULT WINAPI WsRemoveCustomHeader( WS_MESSAGE *handle, const WS_XML_STRING *na
         return E_INVALIDARG;
     }
 
-    if (msg->state < WS_MESSAGE_STATE_INITIALIZED)
+    if (msg->state < WS_MESSAGE_STATE_INITIALIZED) hr = WS_E_INVALID_OPERATION;
+    else
     {
-        LeaveCriticalSection( &msg->cs );
-        return WS_E_INVALID_OPERATION;
-    }
-
-    for (i = 0; i < msg->header_count; i++)
-    {
-        if (msg->header[i]->type || msg->header[i]->mapped) continue;
-        if (WsXmlStringEquals( name, &msg->header[i]->name, NULL ) == S_OK &&
-            WsXmlStringEquals( ns, &msg->header[i]->ns, NULL ) == S_OK)
+        for (i = 0; i < msg->header_count; i++)
         {
-            remove_header( msg, i );
-            removed = TRUE;
-            i--;
+            if (msg->header[i]->type || msg->header[i]->mapped) continue;
+            if (WsXmlStringEquals( name, &msg->header[i]->name, NULL ) == S_OK &&
+                WsXmlStringEquals( ns, &msg->header[i]->ns, NULL ) == S_OK)
+            {
+                remove_header( msg, i );
+                removed = TRUE;
+                i--;
+            }
         }
+        if (removed) hr = write_envelope( msg );
     }
 
-    if (removed) hr = write_envelope( msg );
-
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
@@ -1785,7 +1756,7 @@ HRESULT message_insert_http_headers( WS_MESSAGE *handle, HINTERNET req )
     case WS_ENVELOPE_VERSION_SOAP_1_2:
     {
         static const WCHAR actionW[] = {'a','c','t','i','o','n','=','"'};
-        ULONG len_action = sizeof(actionW)/sizeof(actionW[0]);
+        ULONG len_action = ARRAY_SIZE( actionW );
 
         if (!(len = MultiByteToWideChar( CP_UTF8, 0, (char *)msg->action->bytes, msg->action->length, NULL, 0 )))
             break;
@@ -1813,6 +1784,7 @@ HRESULT message_insert_http_headers( WS_MESSAGE *handle, HINTERNET req )
 done:
     heap_free( header );
     LeaveCriticalSection( &msg->cs );
+    TRACE( "returning %08x\n", hr );
     return hr;
 }
 
