@@ -134,6 +134,7 @@ static const struct object_ops named_pipe_ops =
 
 /* common server and client pipe end functions */
 static void pipe_end_destroy( struct object *obj );
+static struct object_type *pipe_end_get_type( struct object *obj );
 static enum server_fd_type pipe_end_get_fd_type( struct fd *fd );
 static struct fd *pipe_end_get_fd( struct object *obj );
 static struct security_descriptor *pipe_end_get_sd( struct object *obj );
@@ -144,7 +145,7 @@ static int pipe_end_write( struct fd *fd, struct async *async_data, file_pos_t p
 static int pipe_end_flush( struct fd *fd, struct async *async );
 static void pipe_end_get_volume_info( struct fd *fd, unsigned int info_class );
 static void pipe_end_reselect_async( struct fd *fd, struct async_queue *queue );
-static void pipe_end_get_file_info( struct fd *fd, unsigned int info_class );
+static void pipe_end_get_file_info( struct fd *fd, obj_handle_t handle, unsigned int info_class );
 
 /* server end functions */
 static void pipe_server_dump( struct object *obj, int verbose );
@@ -155,7 +156,7 @@ static const struct object_ops pipe_server_ops =
 {
     sizeof(struct pipe_server),   /* size */
     pipe_server_dump,             /* dump */
-    no_get_type,                  /* get_type */
+    pipe_end_get_type,            /* get_type */
     add_queue,                    /* add_queue */
     remove_queue,                 /* remove_queue */
     default_fd_signaled,          /* signaled */
@@ -196,7 +197,7 @@ static const struct object_ops pipe_client_ops =
 {
     sizeof(struct pipe_client),   /* size */
     pipe_client_dump,             /* dump */
-    no_get_type,                  /* get_type */
+    pipe_end_get_type,            /* get_type */
     add_queue,                    /* add_queue */
     remove_queue,                 /* remove_queue */
     default_fd_signaled,          /* signaled */
@@ -270,7 +271,7 @@ static const struct fd_ops named_pipe_device_fd_ops =
     no_fd_read,                       /* read */
     no_fd_write,                      /* write */
     no_fd_flush,                      /* flush */
-    no_fd_get_file_info,              /* get_file_info */
+    default_fd_get_file_info,         /* get_file_info */
     no_fd_get_volume_info,            /* get_volume_info */
     named_pipe_device_ioctl,          /* ioctl */
     default_fd_queue_async,           /* queue_async */
@@ -315,6 +316,13 @@ static void named_pipe_destroy( struct object *obj)
     free_async_queue( &pipe->waiters );
 }
 
+static struct object_type *pipe_end_get_type( struct object *obj )
+{
+    static const WCHAR name[] = {'F','i','l','e'};
+    static const struct unicode_str str = { name, sizeof(name) };
+    return get_object_type( &str );
+}
+
 static struct fd *pipe_end_get_fd( struct object *obj )
 {
     struct pipe_end *pipe_end = (struct pipe_end *) obj;
@@ -493,6 +501,12 @@ static int pipe_end_flush( struct fd *fd, struct async *async )
 {
     struct pipe_end *pipe_end = get_fd_user( fd );
 
+    if (!pipe_end->pipe)
+    {
+        set_error( STATUS_PIPE_DISCONNECTED );
+        return 0;
+    }
+
     if (pipe_end->connection && !list_empty( &pipe_end->connection->message_queue ))
     {
         fd_queue_async( pipe_end->fd, async, ASYNC_TYPE_WAIT );
@@ -501,17 +515,11 @@ static int pipe_end_flush( struct fd *fd, struct async *async )
     return 1;
 }
 
-static void pipe_end_get_file_info( struct fd *fd, unsigned int info_class )
+static void pipe_end_get_file_info( struct fd *fd, obj_handle_t handle, unsigned int info_class )
 {
     struct pipe_end *pipe_end = get_fd_user( fd );
     struct named_pipe *pipe = pipe_end->pipe;
 
-    if (!pipe)
-    {
-        set_error( STATUS_PIPE_DISCONNECTED );
-        return;
-    }
-
     switch (info_class)
     {
     case FileNameInformation:
@@ -526,13 +534,13 @@ static void pipe_end_get_file_info( struct fd *fd, unsigned int info_class )
                 return;
             }
 
-            name = get_object_name( &pipe->obj, &name_len );
             /* FIXME: We should be able to return on unlinked pipe */
-            if (!name)
+            if (!pipe || !(name = get_object_name( &pipe->obj, &name_len )))
             {
                 set_error( STATUS_PIPE_DISCONNECTED );
                 return;
             }
+
             reply_size = offsetof( FILE_NAME_INFORMATION, FileName[name_len/sizeof(WCHAR) + 1] );
             if (reply_size > get_reply_max_size())
             {
@@ -547,8 +555,84 @@ static void pipe_end_get_file_info( struct fd *fd, unsigned int info_class )
             if (reply_size) memcpy( &name_info->FileName[1], name, reply_size );
             break;
         }
+    case FilePipeInformation:
+    {
+            FILE_PIPE_INFORMATION *pipe_info;
+
+            if (!(get_handle_access( current->process, handle) & FILE_READ_ATTRIBUTES))
+            {
+                set_error( STATUS_ACCESS_DENIED );
+                return;
+            }
+
+            if (get_reply_max_size() < sizeof(*pipe_info))
+            {
+                set_error( STATUS_INFO_LENGTH_MISMATCH );
+                return;
+            }
+
+            if (!pipe)
+            {
+                set_error( STATUS_PIPE_DISCONNECTED );
+                return;
+            }
+
+            if (!(pipe_info = set_reply_data_size( sizeof(*pipe_info) ))) return;
+            pipe_info->ReadMode       = (pipe_end->flags & NAMED_PIPE_MESSAGE_STREAM_READ)
+                ? FILE_PIPE_MESSAGE_MODE : FILE_PIPE_BYTE_STREAM_MODE;
+            pipe_info->CompletionMode = (pipe_end->flags & NAMED_PIPE_NONBLOCKING_MODE)
+                ? FILE_PIPE_COMPLETE_OPERATION : FILE_PIPE_QUEUE_OPERATION;
+            break;
+        }
+    case FilePipeLocalInformation:
+        {
+            FILE_PIPE_LOCAL_INFORMATION *pipe_info;
+
+            if (!(get_handle_access( current->process, handle) & FILE_READ_ATTRIBUTES))
+            {
+                set_error( STATUS_ACCESS_DENIED );
+                return;
+            }
+
+            if (get_reply_max_size() < sizeof(*pipe_info))
+            {
+                set_error( STATUS_INFO_LENGTH_MISMATCH );
+                return;
+            }
+
+            if (!pipe)
+            {
+                set_error( STATUS_PIPE_DISCONNECTED );
+                return;
+            }
+
+            if (!(pipe_info = set_reply_data_size( sizeof(*pipe_info) ))) return;
+            pipe_info->NamedPipeType = (pipe_end->flags & NAMED_PIPE_MESSAGE_STREAM_WRITE) != 0;
+            switch (pipe->sharing)
+            {
+            case FILE_SHARE_READ:
+                pipe_info->NamedPipeConfiguration = FILE_PIPE_OUTBOUND;
+                break;
+            case FILE_SHARE_WRITE:
+                pipe_info->NamedPipeConfiguration = FILE_PIPE_INBOUND;
+                break;
+            case FILE_SHARE_READ | FILE_SHARE_WRITE:
+                pipe_info->NamedPipeConfiguration = FILE_PIPE_FULL_DUPLEX;
+                break;
+            }
+            pipe_info->MaximumInstances    = pipe->maxinstances;
+            pipe_info->CurrentInstances    = pipe->instances;
+            pipe_info->InboundQuota        = pipe->insize;
+            pipe_info->ReadDataAvailable   = 0; /* FIXME */
+            pipe_info->OutboundQuota       = pipe->outsize;
+            pipe_info->WriteQuotaAvailable = 0; /* FIXME */
+            pipe_info->NamedPipeState      = pipe_end->state;
+            pipe_info->NamedPipeEnd        = pipe_end->obj.ops == &pipe_server_ops
+                ? FILE_PIPE_SERVER_END : FILE_PIPE_CLIENT_END;
+            break;
+        }
     default:
-        no_fd_get_file_info( fd, info_class );
+        default_fd_get_file_info( fd, handle, info_class );
     }
 }
 
@@ -816,7 +900,7 @@ static int pipe_end_peek( struct pipe_end *pipe_end )
         set_error( STATUS_PIPE_BROKEN );
         return 0;
     default:
-        set_error( STATUS_INVALID_PIPE_STATE );
+        set_error( pipe_end->pipe ? STATUS_INVALID_PIPE_STATE : STATUS_PIPE_DISCONNECTED );
         return 0;
     }
 
@@ -849,6 +933,7 @@ static int pipe_end_peek( struct pipe_end *pipe_end )
             if (write_pos == reply_size) break;
         }
     }
+    if (message_length > reply_size) set_error( STATUS_BUFFER_OVERFLOW );
     return 1;
 }
 
@@ -859,7 +944,7 @@ static int pipe_end_transceive( struct pipe_end *pipe_end, struct async *async )
 
     if (!pipe_end->connection)
     {
-        set_error( STATUS_INVALID_PIPE_STATE );
+        set_error( pipe_end->pipe ? STATUS_INVALID_PIPE_STATE : STATUS_PIPE_DISCONNECTED );
         return 0;
     }
 
@@ -1271,39 +1356,6 @@ DECL_HANDLER(create_named_pipe)
     release_object( pipe );
 }
 
-DECL_HANDLER(get_named_pipe_info)
-{
-    struct pipe_end *pipe_end;
-
-    pipe_end = (struct pipe_end *)get_handle_obj( current->process, req->handle,
-                                                  FILE_READ_ATTRIBUTES, &pipe_server_ops );
-    if (!pipe_end)
-    {
-        if (get_error() != STATUS_OBJECT_TYPE_MISMATCH)
-            return;
-
-        clear_error();
-        pipe_end = (struct pipe_end *)get_handle_obj( current->process, req->handle,
-                                                      FILE_READ_ATTRIBUTES, &pipe_client_ops );
-        if (!pipe_end) return;
-    }
-
-    if (pipe_end->pipe)
-    {
-        reply->flags        = pipe_end->flags;
-        reply->sharing      = pipe_end->pipe->sharing;
-        reply->maxinstances = pipe_end->pipe->maxinstances;
-        reply->instances    = pipe_end->pipe->instances;
-        reply->insize       = pipe_end->pipe->insize;
-        reply->outsize      = pipe_end->pipe->outsize;
-
-        if (pipe_end->obj.ops == &pipe_server_ops) reply->flags |= NAMED_PIPE_SERVER_END;
-    }
-    else set_error( STATUS_PIPE_DISCONNECTED );
-
-    release_object( pipe_end );
-}
-
 DECL_HANDLER(set_named_pipe_info)
 {
     struct pipe_end *pipe_end;
